package SecureProtocol;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.math.BigInteger;
import java.util.LinkedList;

import SecureProtocol.BasicPrimitives.OT_N_1Server;
import utils.crypto.ElGamalEncryption;
import utils.crypto.HomomorphicInstance;
import utils.crypto.HomomorphicInstance.HomomorphicPrivateKey;
import utils.crypto.HomomorphicInstance.HomomorphicPublicKey;


public class BinHDOTClient {
	
	// Client image
	private boolean image[]=null;
	
	// Random binary words
	private boolean randomCWord[]=null;
	
	// Homomorphic encryptions of 0 and 1
	private HomomorphicInstance[] randomWords=null;
	
	// Length of the image representive vector.
	final static private int imageLength=900; 
	
	private int DBSize=9;
	
	// Homomorphic private and public Key
	final private static HomomorphicPrivateKey homomorphicPrivateKey=new HomomorphicInstance.HomomorphicPrivateKey();
	final private static HomomorphicPublicKey homomorphicPublicKey=homomorphicPrivateKey.getPublicKey();
	
	// OT clients
	private OT_N_1Server[] ots=null;
	
	// Input and output buffers
	private BufferedReader br=null;
	private BufferedWriter bw=null;	
	
	public BinHDOTClient(
			BufferedReader br,
			BufferedWriter bw,
			int DBSize) {
		
		if(DBSize<1) {
			System.err.println("BinHDOTCient, constructor: data base size should be positive.");
			System.exit(-1);
		}
		if(br==null) {
			System.err.println("BinHDOTCient, constructor: null BufferedReader.");
			System.exit(-1);
		}
		if(bw==null) {
			System.err.println("BinHDOTCient, constructor: null BufferedWriter.");
			System.exit(-1);
		}
		this.br=br;
		this.bw=bw;
		this.DBSize=DBSize;
	}
	
	private void prepareRandomVector() {
		randomCWord=new boolean[imageLength];
		randomWords=new HomomorphicInstance[imageLength];
		for(int i=0; i<imageLength; i++) {
			randomCWord[i]=ElGamalEncryption.RAND_GENERATOR.nextBoolean();			
			
			if(randomCWord[i]){
				randomWords[i]=HomomorphicInstance.getOne(homomorphicPublicKey);
			} else {
				randomWords[i]=HomomorphicInstance.getZero(homomorphicPublicKey);
			}
			
		}
	}
	
	private void sendRandomEncryptedWord() {
		try {
			for(int i=0; i<imageLength; i++) {
				bw.write(randomWords[i].toString());
				bw.newLine();
			}
			bw.flush();
		} catch(IOException ioe) {
			System.err.println("BinHDOTCient, sendRandomEncryptedWord: "+ioe.getMessage());
			System.exit(-1);
		}
	}
	
	
	public void preprocessing() {
		prepareRandomVector();
		sendRandomEncryptedWord();
		
		ots=new OT_N_1Server[DBSize];
		for(int i=0; i<DBSize; i++) {
			ots[i]=new OT_N_1Server(br, bw);
			ots[i].preprocessing(imageLength);
		}
		
	}
	
	private void sendFixingBits() {
		Integer o=Integer.valueOf(1);
		Integer z=Integer.valueOf(0);
		try {
			for(int i=0; i<imageLength; i++) {
				if(image[i] ^ randomCWord[i]) {
					bw.write(o.toString());
				} else {
					bw.write(z.toString());
				}
				bw.newLine();
			}
			bw.flush();
		} catch(IOException ioe) {
			System.err.println("BinHDOTCient, sendFixingBits: "+ioe.getMessage());
			System.exit(-1);
		}
	}
	
	public BigInteger readHammingDistance() {
		try {
			HomomorphicInstance hi=new HomomorphicInstance(homomorphicPublicKey, br.readLine());
			BigInteger temp=HomomorphicInstance.decrypt(homomorphicPrivateKey, hi).mod(new BigInteger("900"));
			return (temp);
		}
		catch(IOException ioe) {
			System.err.println("BinHDOTClient, readHammingDistance: "+ioe.getMessage());
			System.exit(-1);
		}
		return null;
	}
	
	public void executeAfterPreprocessing(boolean[] word) {
		LinkedList<BigInteger> hdList=new LinkedList<BigInteger>();
		image=word;
		sendFixingBits();
		for(int i=0; i<DBSize; i++) {
			hdList.add(readHammingDistance());
		}
		
		for(int i=0; i<DBSize ; i++) {
			BigInteger hd=hdList.get(i);
			LinkedList<BigInteger> sList=new LinkedList<BigInteger>();
			for(int j=0; j<imageLength; j++) {
				if((j>=((hd.mod(new BigInteger(""+imageLength)).intValue()-180)%900)) && (j<((hd.intValue())%900))) {
					sList.add(BigInteger.ZERO);
				}
				else {
					sList.add(BigInteger.ONE);
				}
			}
			ots[i].executeAfterPreprocessing(sList);
		}
		
	}
	
	
}
