package SecureProtocol;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.math.BigInteger;
import java.util.LinkedList;

import javax.swing.JTextArea;

import SecureProtocol.BasicPrimitives.OT_N_1Client;
import utils.crypto.ElGamalEncryption;
import utils.crypto.HomomorphicInstance;
import utils.crypto.HomomorphicInstance.HomomorphicPublicKey;


public class BinHDOTServer extends Thread {
	
	JTextArea jtServer=null;
	
	// Length of the image representive vector.
	final static private int imageLength=900; 
	
	// Homomorphic public Key
	final private static HomomorphicPublicKey homomorphicPublicKey=new HomomorphicInstance.HomomorphicPrivateKey().getPublicKey();
	
	// Client fixing bits
	private boolean fixingCWord[]=null; 
	
	// Input and output buffers
	private BufferedReader br=null;
	private BufferedWriter bw=null;

	// The Encrypted binary values that the client sends to the server 
	private HomomorphicInstance cWord[]=null;
	
	// OT client
	private OT_N_1Client[] otc=null;
	
	private LinkedList<boolean[]> DB=null;
	private int[] thresholds=null;
	private int DBSize=9;
	
	// Random numbers
	private BigInteger r[]=null;
	
	// Possible sums
	private LinkedList<HomomorphicInstance[]> possibleSums=null;
	
	
	public BinHDOTServer(JTextArea jf,
			BufferedReader br,
			BufferedWriter bw,
			LinkedList<boolean[]> DB,
			int t[]) {	
		this.jtServer=jf;
		if(DB==null) {
			System.err.println("BinHDOTServer, constructor: null db.");
			System.exit(-1);
		}
		if(DB.isEmpty()) {
			System.err.println("BinHDOTServer, constructor: empty db.");
			System.exit(-1);
		}
		if(br==null) {
			System.err.println("BinHDOTServer, constructor: null BufferedReader.");
			System.exit(-1);
		}
		if(bw==null) {
			System.err.println("BinHDOTServer, constructor: null BufferedWriter.");
			System.exit(-1);
		}
		this.br=br;
		this.bw=bw;
		this.DB=DB;
		this.DBSize=DB.size();
		thresholds=new int[DBSize];
		for(int i=0; i<DBSize; i++) {
			thresholds[i]=t[i];
		}
		
	}
	
	private void readClientEncryptedWord() {
		cWord=new HomomorphicInstance[imageLength];
		try {
			for(int i=0; i<imageLength; i++) {
				cWord[i]=new HomomorphicInstance(homomorphicPublicKey, br.readLine());
			}
		}
		catch(IOException ioe) {
			System.err.println("BinHDOTServer, readClientEncryptedWord: "+ioe.getMessage());
			System.exit(-1);
		}
	}
	
	public void preparePossibleCounts() {
		possibleSums=new LinkedList<HomomorphicInstance[]>();
		
		HomomorphicInstance o=HomomorphicInstance.getOne(homomorphicPublicKey);
		for(int i=0; i<imageLength/2; i++) {
			HomomorphicInstance[] sum=new HomomorphicInstance[4];
			HomomorphicInstance arr[]=new HomomorphicInstance[2];
			HomomorphicInstance cArr[]=new HomomorphicInstance[2];
			
			arr[0]=cWord[2*i];
			cArr[0]=o.substract(cWord[2*i]);
			arr[1]=cWord[2*i+1];
			cArr[1]=o.substract(cWord[2*i+1]);


			//00
			sum[0]=arr[0].add(arr[1]);
			
			//10
			sum[1]=cArr[0].add(arr[1]);
			
			//01
			sum[2]=arr[0].add(cArr[1]);
			
			//11
			sum[3]=cArr[0].add(cArr[1]);
			
			possibleSums.add(sum);	
		}
	}
	
	public void preprocessing() {
		String txt=jtServer.getText();
		txt=txt+"Start Preprocessing\n";
		jtServer.setText(txt);
		r=new BigInteger[DBSize];
		
		for(int i=0; i<DBSize; i++) {
			r[i]=ElGamalEncryption.getRandom();
		}
		
		fixingCWord=new boolean[imageLength];
		for(int i=0; i<fixingCWord.length; i++) {
			fixingCWord[i]=false;
		}
		txt=txt+"Read client encrypted random input\n";
		jtServer.setText(txt);
		readClientEncryptedWord();
		
		preparePossibleCounts();
		txt=txt+"Initalize OTs \n";
		jtServer.setText(txt);
		otc=new OT_N_1Client[DBSize];
		for(int i=0; i<DBSize; i++) {
			otc[i]=new OT_N_1Client(br, bw);
			otc[i].preprocessing((r[i].mod(new BigInteger(""+imageLength)).intValue()+thresholds[i])%imageLength, imageLength);
		}
		
	}
		
	
	private LinkedList<HomomorphicInstance> computeHammingDistance() {
		LinkedList<HomomorphicInstance> finalSumList=new LinkedList<HomomorphicInstance>();
		for(int i=0; i<DBSize; i++) {
			boolean[] sWord=DB.get(i);
			HomomorphicInstance finalSum=HomomorphicInstance.getZero(homomorphicPublicKey);
			for(int j=0; j<imageLength/2; j++) {
				int index=0;
				if((fixingCWord[2*j] ^ sWord[2*j])) {
					index+=1;
				}
				if((fixingCWord[2*j+1] ^ sWord[2*j+1])) {
					index+=2;
				}
				finalSum=finalSum.add(possibleSums.get(j)[index]);
			}
			finalSumList.add(finalSum);
		}
		
		return finalSumList;
	}
	
	private void sendHammingDistance(HomomorphicInstance hi) {
		try {
			bw.write(hi.toString());
			bw.newLine();
			bw.flush();
		} catch (IOException ioe) {
			System.err.println("BinHDOTServer, sendHammingDistance: "+ioe.getMessage());
			System.exit(-1);
		}
		
	}
	
	
	private void readFixingBits() {
		fixingCWord=new boolean[imageLength];
		try {
			int temp;
			for(int i=0; i<imageLength; i++) {
				String temp1=br.readLine();
				temp=Integer.valueOf(temp1).intValue();
				if(temp==1) {
					fixingCWord[i]=true;
				} else {
					fixingCWord[i]=false;
				}
			}
		}
		catch(IOException ioe) {
			System.err.println("BinHDOTServer, readClientEncryptedWord: "+ioe.getMessage());
			System.exit(-1);
		}
	}
	
	public LinkedList<BigInteger> executeAfterPreprocessing() {
		String txt=jtServer.getText();
		txt=txt+"Start Execution\n";
		jtServer.setText(txt);
		LinkedList<BigInteger> resList=new LinkedList<BigInteger>();
		readFixingBits();
		txt=txt+"Read fixing bits\n";
		jtServer.setText(txt);
		txt=txt+"Compute Hamming distance\n";
		jtServer.setText(txt);
		LinkedList<HomomorphicInstance> hdList=computeHammingDistance();
		for(int i=0; i<DBSize; i++) {
			sendHammingDistance(hdList.get(i).add(r[i]));
		}
		txt=txt+"Execute OTs\n";
		jtServer.setText(txt);
		for(int i=0; i<DBSize; i++) {
			resList.add(otc[i].executeAfterPreprocessing());
		}
		return resList;
	}
}
