package UI;

import javax.imageio.ImageIO;
import javax.swing.ImageIcon;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JLabel;

import java.awt.Dimension;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.event.WindowEvent;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.math.BigInteger;
import java.nio.ByteBuffer;
import java.util.LinkedList;

import javax.swing.JButton;

import com.mathworks.toolbox.javabuilder.MWException;
import buildFaceRep_fromFile.buildFaceRep_fromFile;

import DB.serverImages;
import au.edu.jcu.v4l4j.FrameGrabber;
import au.edu.jcu.v4l4j.V4L4JConstants;
import au.edu.jcu.v4l4j.VideoDevice;
import au.edu.jcu.v4l4j.exceptions.V4L4JException;

public class ClientCamViewer extends JFrame implements Runnable {
	
	private static final long serialVersionUID = 1L;
	
	private VideoDevice vd; 
	private FrameGrabber fg;
	private Thread captureThread;
    private boolean stop;

	private JPanel jContentPane = null;
	private JLabel camLabal = null;
	private JButton captureButton = null;
	private JButton CheckButton = null;
	private ByteBuffer frameBuffer=null;
	private JButton ProceedButton = null;
	private String[] imageNameList=null;  
	private boolean[][] imageVecList=null;
	private int[] imageThresholdList=null;
	private byte[] b=null;

	/**
	 * This is the default constructor
	 */
	public ClientCamViewer(String dev, int w, int h, int std, int channel, int qty) {
		super();
		initializeFrameGrabber(dev, w, h, std, channel, qty);
		
		initializeGUI();
		stop = false;
        captureThread = new Thread(this, "Capture Thread");
        captureThread.start();
        readDB();
	}
	
	private void readDB() {
		LinkedList<String> tempList=null;
		tempList=serverImages.readFileDB("./DB/imageList.txt");
		
		for(int i=tempList.size()-1; i>=0; i--) {
			String s=tempList.get(i);
			if(s.length()<900) tempList.remove(i);
		}
		
		if(!tempList.isEmpty()) {
			imageNameList=new String[tempList.size()];
			imageVecList=new boolean[tempList.size()][];
			imageThresholdList=new int[tempList.size()];
		} else {
			return;
		}
		
		for(int i=0; i<tempList.size(); i++) {
			boolean image[]=new boolean[900];
			String s=tempList.get(i);
			int space1=s.indexOf(' ');
			
			if(s.length()<900) continue;
			imageNameList[i]=new String(s.substring(0, space1));
			s=s.substring(space1+1);
			int space2=s.indexOf(' ');
			for(int j=0;j<space2;j++) {
				if(s.charAt(j)=='0') image[j]=false;
				if(s.charAt(j)=='1') image[j]=true;
			}
			
			imageVecList[i]=image;
			
			s=s.substring(space2+1);
			imageThresholdList[i]=Integer.valueOf(s);
		}
		
	}
	
	/**
	 * This method initializes this
	 * 
	 */
	private void initializeGUI() {
		this.setSize(670, 620);
		this.setResizable(false);
		this.setContentPane(getJContentPane());
		this.CheckButton.setEnabled(false);
		this.ProceedButton.setEnabled(false);
		this.addWindowListener(new java.awt.event.WindowAdapter() {
        	@SuppressWarnings("deprecation")
			public void windowClosing(java.awt.event.WindowEvent e) {        	
        		if(captureThread.isAlive()){
                    stop = true;
                    captureThread.stop();
            }
            
            fg.stopCapture();
            vd.releaseFrameGrabber();
            vd.release(true);
            System.exit(-1);
        	}
        });
		this.setVisible(true);
		this.setTitle("Client Viewer");
	}
	
	 /**
     * Initializes the FrameGrabber object with the given parameters
         * @param dev the video device file to capture from
         * @param w the desired capture width
         * @param h the desired capture height
         * @param std the capture standard
         * @param channel the capture channel
         * @param qty the JPEG compression quality
     */
    private void initializeFrameGrabber(String dev, int w, int h, int std, int channel, int qty){
    	try {
			vd = new VideoDevice(dev);
			fg = vd.getJPEGFrameGrabber(w, h, channel, std, qty);
	        fg.startCapture();
		} catch (V4L4JException v4v4je) {
			v4v4je.printStackTrace();
			System.exit(-1);
		}         
    }

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			camLabal = new JLabel();
			camLabal.setBounds(new Rectangle(13, 12, 640, 480));
			camLabal.setText("JLabel");
			jContentPane = new JPanel();
			jContentPane.setLayout(null);
			jContentPane.setSize(new Dimension(670, 570));
			jContentPane.add(camLabal, null);
			jContentPane.add(getCaptureButton(), null);
			jContentPane.add(getCheckButton(), null);
			jContentPane.add(getProceedButton(), null);
		}
		return jContentPane;
	}

	/**
	 * This method initializes captureButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getCaptureButton() {
		if (captureButton == null) {
			captureButton = new JButton();
			captureButton.setBounds(new Rectangle(15, 520, 140, 40));
			captureButton.setText("Capture");
			captureButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					captureButton.setEnabled(false);
					CheckButton.setEnabled(true);
					ProceedButton.setEnabled(true);
					synchronized (this) {
						stop=true;
					}
				}
			});
		}
		return captureButton;
	}

	private void tempCompare() {
		buildFaceRep_fromFile bf=null;
		LinkedList<String> out=null;
		boolean flagNoFound=true;
		try {
			
			BufferedImage imag=ImageIO.read(new ByteArrayInputStream(b));
			ImageIO.write(imag, "jpeg", new File("./images/image.jpeg"));
			bf=new buildFaceRep_fromFile();
			Object[] outL=new Object[1];
			bf.buildFaceRep_fromFile(outL,new Object[0]);
			
			
			BigInteger mOut=new BigInteger(outL[0].toString());
			if(mOut.intValue()==0) return;

			
			bf.dispose();
			
			Object[] option={"Yes","No"};
			int n=JOptionPane.showOptionDialog(this,"Continue?","",JOptionPane.YES_NO_OPTION,JOptionPane.QUESTION_MESSAGE,null,option,option[0]);
			if(n==1) return;
			
			out = serverImages.readFileDB("./reps/v.txt");
		
		} catch (MWException mwe) {
			// TODO Auto-generated catch block
			mwe.printStackTrace();
			return;
		} catch (IOException ioe) {
			// TODO Auto-generated catch block
			ioe.printStackTrace();
			return;
		}
		boolean image[]=new boolean[900];
		for(int i=0; i<900; i++) {
			if(out.get(i).charAt(0)=='1') image[i]=true;
			else image[i]=false;
		}
			
		for(int i=0; i<imageNameList.length; i++) {
			int counter=0;
			for(int j=0; j<900; j++) {
				if(image[j]!=imageVecList[i][j]) counter++;
			}
			
			if(counter<imageThresholdList[i]) {
				JOptionPane.showMessageDialog(this,"Image found: "+imageNameList[i]);
				flagNoFound=false;
			}
		}
		if(flagNoFound) JOptionPane.showMessageDialog(this,"No image found");
	}
	
	/**
	 * This method initializes CheckButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getCheckButton() {
		if (CheckButton == null) {
			CheckButton = new JButton();
			CheckButton.setBounds(new Rectangle(175, 520, 140, 40));
			CheckButton.setText("Check");
			CheckButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					captureButton.setEnabled(false);
					CheckButton.setEnabled(false);
					ProceedButton.setEnabled(false);
					tempCompare();
					captureButton.setEnabled(true);
					CheckButton.setEnabled(false);
					ProceedButton.setEnabled(false);
					synchronized (frameBuffer) {
						stop=false;
						frameBuffer.notifyAll();
					}
				}
			});
		}
		return CheckButton;
	}

	

	
	public void run(){
        
        try { 
        	while(true) {
	        	while(!stop){
	        		frameBuffer = fg.getFrame();
	        		b = new byte[frameBuffer.limit()];
	        		frameBuffer.get(b);
	        		camLabal.setIcon(new ImageIcon(new ImageIcon(b).getImage().getScaledInstance(640, 480, Image.SCALE_DEFAULT)));
	        	}
	        	frameBuffer = fg.getFrame();
        		b = new byte[frameBuffer.limit()];
        		frameBuffer.get(b);
        		camLabal.setIcon(new ImageIcon(new ImageIcon(b).getImage().getScaledInstance(640, 480, Image.SCALE_DEFAULT)));
				synchronized (frameBuffer) {
					while(stop) {
						frameBuffer.wait();						
					}
				}
				
        	}
        } catch (V4L4JException v4v4je) {
        	v4v4je.printStackTrace();
        	System.err.println("Failed to capture image");
        } catch (InterruptedException ie) {
			// TODO Auto-generated catch block
			ie.printStackTrace();
		}
        
    }
	
	public void windowClosing(WindowEvent e) {
        if(captureThread.isAlive()){
                stop = true;
                try {
                	captureThread.join();
                } catch (InterruptedException ie) {
                	ie.printStackTrace();
                }
        }
        
        fg.stopCapture();
        vd.releaseFrameGrabber();
}
	
	/**
	 * This method initializes ProceedButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getProceedButton() {
		if (ProceedButton == null) {
			ProceedButton = new JButton();
			ProceedButton.setBounds(new Rectangle(335, 520, 140, 40));
			ProceedButton.setText("Proceed");
			ProceedButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					captureButton.setEnabled(true);
					CheckButton.setEnabled(false);
					ProceedButton.setEnabled(false);
					synchronized (frameBuffer) {
						stop=false;
						frameBuffer.notifyAll();
					}
				}
			});
		}
		return ProceedButton;
	}

	public static void main(String[] args) {

        String dev = CamSrc.dev;
        int w=V4L4JConstants.MAX_WIDTH, h=V4L4JConstants.MAX_HEIGHT, std=V4L4JConstants.STANDARD_WEBCAM , channel = 0, qty = 60;
        new ClientCamViewer(dev,w,h,std,channel,qty);
}
}  //  @jve:decl-index=0:visual-constraint="10,10"