package UI;


import javax.imageio.ImageIO;
import javax.swing.ImageIcon;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JLabel;

import java.awt.Image;
import java.awt.Rectangle;
import java.awt.event.WindowEvent;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.math.BigInteger;
import java.nio.ByteBuffer;
import java.util.LinkedList;

import javax.swing.JButton;

import com.mathworks.toolbox.javabuilder.MWException;

import buildFaceRep_fromFile.buildFaceRep_fromFile;

import DB.ThresholdConstant;
import DB.serverImages;
import au.edu.jcu.v4l4j.FrameGrabber;
import au.edu.jcu.v4l4j.V4L4JConstants;
import au.edu.jcu.v4l4j.VideoDevice;
import au.edu.jcu.v4l4j.exceptions.V4L4JException;
import java.awt.Dimension;
import javax.swing.JTextField;

public class SaveToDB extends JFrame implements Runnable {
	
	private static final long serialVersionUID = 1L;
	
	private VideoDevice vd;  
	private FrameGrabber fg; 
	private Thread captureThread;
    private boolean stop;

	
	private JPanel jContentPane = null;
	private JLabel camLabal = null;
	private JButton captureButton = null;
	private JButton SaveButton = null;
	private ByteBuffer frameBuffer=null;
	private JButton ProceedButton = null;
	private JTextField jTextField = null;
	String[] imageNameList=null; 
	boolean[][] imageVecList=null;
	int[] imageThresholdList=null;
	private byte[] b=null;

	/**
	 * This is the default constructor
	 */
	public SaveToDB(String dev, int w, int h, int std, int channel, int qty) {
		super();
		initializeFrameGrabber(dev, w, h, std, channel, qty);
		
		initializeGUI();
		stop = false;
        captureThread = new Thread(this, "Capture Thread");
        captureThread.start();
        readDB();
	}
	
	private void readDB() {
		LinkedList<String> tempList=null;
		tempList=serverImages.readFileDB("./DB/imageList.txt");
		
		for(int i=tempList.size()-1; i>=0; i--) {
			String s=tempList.get(i);
			if(s.length()<900) tempList.remove(i);
		}
		if(!tempList.isEmpty()) {
			imageNameList=new String[tempList.size()];
			imageVecList=new boolean[tempList.size()][];
			imageThresholdList=new int[tempList.size()];
		} else {
			return;
		}
		
		for(int i=0; i<tempList.size(); i++) {
			boolean image[]=new boolean[900];
			String s=tempList.get(i);
			if(s.length()<900) continue;
			int space1=s.indexOf(' ');
			
			imageNameList[i]=new String(s.substring(0, space1));
			s=s.substring(space1+1);
			int space2=s.indexOf(' ');
			for(int j=0;j<space2;j++) {
				if(s.charAt(j)=='0') image[j]=false;
				if(s.charAt(j)=='1') image[j]=true;
			}
			
			imageVecList[i]=image;			
			s=s.substring(space2+1);
			imageThresholdList[i]=Integer.valueOf(s);
		}
		
	}
	
	/**
	 * This method initializes this
	 * 
	 */
	private void initializeGUI() {
		
		this.setSize(670, 620);
		this.setResizable(false);
		this.setContentPane(getJContentPane());
		this.SaveButton.setEnabled(false);
		this.ProceedButton.setEnabled(false);
		this.addWindowListener(new java.awt.event.WindowAdapter() {
        	@SuppressWarnings("deprecation")
			public void windowClosing(java.awt.event.WindowEvent e) {
        		if(captureThread.isAlive()){
                    stop = true;
                    captureThread.stop();
            }
            fg.stopCapture();
            vd.releaseFrameGrabber();
            System.exit(-1);
        	}
        });
		this.setVisible(true);
		this.setTitle("DB");
	}
	
	 /**
     * Initializes the FrameGrabber object with the given parameters
         * @param dev the video device file to capture from
         * @param w the desired capture width
         * @param h the desired capture height
         * @param std the capture standard
         * @param channel the capture channel
         * @param qty the JPEG compression quality
     */
    private void initializeFrameGrabber(String dev, int w, int h, int std, int channel, int qty){
    	try {
			vd = new VideoDevice(dev);
			
			fg = vd.getJPEGFrameGrabber(w, h, channel, std, qty);
			fg.startCapture();
		} catch (V4L4JException v4l4je) {
			v4l4je.printStackTrace();
			System.exit(-1);
		}         
    }

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			camLabal = new JLabel();
			camLabal.setBounds(new Rectangle(13, 12, 640, 480));
			camLabal.setText("JLabel");
			jContentPane = new JPanel();
			jContentPane.setLayout(null);
			jContentPane.setSize(new Dimension(670, 570));
			jContentPane.add(camLabal, null);
			jContentPane.add(getCaptureButton(), null);
			jContentPane.add(getSaveButton(), null);
			jContentPane.add(getProceedButton(), null);
			jContentPane.add(getJTextField(), null);
		}
		return jContentPane;
	}

	/**
	 * This method initializes captureButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getCaptureButton() {
		if (captureButton == null) {
			captureButton = new JButton();
			captureButton.setBounds(new Rectangle(15, 520, 140, 40));
			captureButton.setText("Capture");
			captureButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					captureButton.setEnabled(false);
					SaveButton.setEnabled(true);
					ProceedButton.setEnabled(true);
					synchronized (this) {
						stop=true;
					}
				}
			});
		}
		return captureButton;
	}
	
	private int computeHD(boolean[] a, boolean[] b) {
		int h=0;
		for(int i=0; i<a.length; i++) {
			if(a[i]!=b[i]) h++;
		}
		return h;
	}
	
	private void save() {
		String fileStr="";
		try {
			String strName=jTextField.getText();
			if(strName==null || strName.equals("") || strName.contains(" ")) {
				JOptionPane.showMessageDialog(this, "Incorrect image name.");
				return;
			}
			BufferedImage imag=ImageIO.read(new ByteArrayInputStream(b));
			ImageIO.write(imag, "jpeg", new File("./images/image.jpeg"));
			
			buildFaceRep_fromFile bf=null;
			bf=new buildFaceRep_fromFile();
			
			Object[] outL=new Object[1];
			bf.buildFaceRep_fromFile(outL,new Object[0]);
			BigInteger mOut=new BigInteger(outL[0].toString());
			if(mOut.intValue()==0) return;
			bf.dispose();
		
			Object[] option={"Yes","No"};
			int n=JOptionPane.showOptionDialog(this,"Continue?","",JOptionPane.YES_NO_OPTION,JOptionPane.QUESTION_MESSAGE,null,option,option[0]);
			if(n==1) return;
			
			LinkedList<String> out = serverImages.readFileDB("./reps/v.txt");
			
			
			boolean img[]=new boolean[900];
			if(strName==null || strName.equals("")) strName="image";
			
			for(int i=0; i<out.size(); i++) {
				String s=out.get(i);
				if(!(s.equals("0") || s.equals("1"))) continue;
				if(s.equals("0")) {img[i]=false; continue;}
				if(s.equals("1")) {img[i]=true; continue;}
			}
			
			if(imageNameList==null) {
				imageNameList=new String[1];
				imageNameList[0]=strName;
				imageVecList=new boolean[1][];
				imageVecList[0]=img;
				imageThresholdList=new int[1];
				imageThresholdList[0]=300;
				
				fileStr=imageNameList[0]+" ";
				for(int i=0; i<900; i++) {
					if(imageVecList[0][i]) fileStr=fileStr+"1";
					else fileStr=fileStr+"0";
				}
				fileStr=fileStr+" "+imageThresholdList[0];
				if(fileStr.length()<900) fileStr="";
				else ImageIO.write(imag, "jpeg", new File("./DB/"+strName+".jpeg"));
			} else if((imageNameList.length==1) && (strName.equalsIgnoreCase(imageNameList[0]))){
				imageNameList=new String[1];
				imageNameList[0]=strName;
				imageVecList=new boolean[1][];
				imageVecList[0]=img;
				imageThresholdList=new int[1];
				imageThresholdList[0]=300;
				
				fileStr=imageNameList[0]+" ";
				for(int i=0; i<900; i++) {
					if(imageVecList[0][i]) fileStr=fileStr+"1";
					else fileStr=fileStr+"0";
				}
				fileStr=fileStr+" "+imageThresholdList[0];
				if(fileStr.length()<900) fileStr="";
				else ImageIO.write(imag, "jpeg", new File("./DB/"+strName+".jpeg"));
			} else {
				int existFlag=-1;
				for(int i=0; i<imageNameList.length; i++) {
					if(strName.equalsIgnoreCase(imageNameList[i])) {
						existFlag=i;
						break;
					}
				}
				if(existFlag!=-1) {
					imageVecList[existFlag]=img;
				} else {
					String[] tempImageNameList=new String[imageNameList.length+1];
					boolean[][] tempImageVecList=new boolean[imageNameList.length+1][];
					int[] tempImageThresholdList=new int[imageThresholdList.length+1];
					for(int i=0; i<imageNameList.length; i++) {
						tempImageNameList[i]=imageNameList[i];
						tempImageVecList[i]=imageVecList[i];
					}
					tempImageNameList[tempImageNameList.length-1]=strName;
					tempImageVecList[tempImageVecList.length-1]=img;
					
					imageNameList=tempImageNameList;
					imageVecList=tempImageVecList;
					imageThresholdList=tempImageThresholdList;
				}
				
				for(int i=0;i<imageNameList.length; i++) {
					int hd=900;
					for(int j=0;j<imageNameList.length; j++) {
						if(i==j) continue;
						int h=computeHD(imageVecList[i], imageVecList[j]);
						hd=(h<hd)?h:hd;
					}
					imageThresholdList[i]=(int) Math.ceil(hd*ThresholdConstant.C);
					if(i!=899) {
						fileStr=fileStr+imageNameList[i]+" ";
						for(int j=0; j<900; j++) {
							if(imageVecList[i][j]) fileStr=fileStr+"1";
							else fileStr=fileStr+"0";
						}
						fileStr=fileStr+" "+imageThresholdList[i]+System.getProperty("line.separator");
					}
					else {
						fileStr=imageNameList[i]+" ";
						for(int j=0; j<900; j++) {
							if(imageVecList[i][j]) fileStr=fileStr+"1";
							else fileStr=fileStr+"0";
						}
						fileStr=fileStr+" "+imageThresholdList[i];
					}
				}
				ImageIO.write(imag, "jpeg", new File("./DB/"+strName+".jpeg"));
			}
			
			DB.serverImages.writeFileDB("./DB/imageList.txt", fileStr, false);
			
			
			
		} catch (MWException mwe) {
			// TODO Auto-generated catch block
			mwe.printStackTrace();
		} catch (IOException ioe) {
			// TODO Auto-generated catch block
			ioe.printStackTrace();
		}
	}

	/**
	 * This method initializes SaveButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getSaveButton() {
		if (SaveButton == null) {
			SaveButton = new JButton();
			SaveButton.setBounds(new Rectangle(175, 520, 140, 40));
			SaveButton.setText("Save to DB");
			SaveButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					captureButton.setEnabled(false);
					SaveButton.setEnabled(false);
					ProceedButton.setEnabled(false);
					save();
					
					captureButton.setEnabled(true);
					SaveButton.setEnabled(false);
					ProceedButton.setEnabled(false);
					synchronized (frameBuffer) {
						stop=false;
						frameBuffer.notifyAll();
					}
				}
			});
		}
		return SaveButton;
	}

	

	
	public void run(){
        
        try { 
        	while(true) {
	        	while(!stop){
	        		frameBuffer = fg.getFrame();
	        		b = new byte[frameBuffer.limit()];
	        		frameBuffer.get(b);
	        		camLabal.setIcon(new ImageIcon(new ImageIcon(b).getImage().getScaledInstance(640, 480, Image.SCALE_DEFAULT)));
	        	}
	        	frameBuffer = fg.getFrame();
        		b = new byte[frameBuffer.limit()];
        		frameBuffer.get(b);
        		camLabal.setIcon(new ImageIcon(new ImageIcon(b).getImage().getScaledInstance(640, 480, Image.SCALE_DEFAULT)));                
        		
				synchronized (frameBuffer) {
					while(stop) {
						frameBuffer.wait();						
					}
				}
				
        	}
        } catch (V4L4JException v4l4je) {
        	v4l4je.printStackTrace();
        	System.err.println("Failed to capture image");
        } catch (InterruptedException ie) {
			// TODO Auto-generated catch block
			ie.printStackTrace();
		}
        
    }
	
	public void windowClosing(WindowEvent e) {
        if(captureThread.isAlive()){
                stop = true;
                try {
                        captureThread.join();
                } catch (InterruptedException ie) {
                	ie.printStackTrace();
                }
        }
        
        fg.stopCapture();
        vd.releaseFrameGrabber();
	}
	
	/**
	 * This method initializes ProceedButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getProceedButton() {
		if (ProceedButton == null) {
			ProceedButton = new JButton();
			ProceedButton.setBounds(new Rectangle(335, 520, 140, 40));
			ProceedButton.setText("Proceed");
			ProceedButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					captureButton.setEnabled(true);
					SaveButton.setEnabled(false);
					ProceedButton.setEnabled(false);
					synchronized (frameBuffer) {
						stop=false;
						frameBuffer.notifyAll();
					}
				}
			});
		}
		return ProceedButton;
	}

	/**
	 * This method initializes jTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getJTextField() {
		if (jTextField == null) {
			jTextField = new JTextField();
			jTextField.setBounds(new Rectangle(491, 520, 160, 26));
			jTextField.setText("");
		}
		return jTextField;
	}

	public static void main(String[] args) {
		
        String dev = CamSrc.dev;
        int w=V4L4JConstants.MAX_WIDTH, h=V4L4JConstants.MAX_HEIGHT, std=V4L4JConstants.STANDARD_WEBCAM , channel = 0, qty = 100;
        new SaveToDB(dev,w,h,std,channel,qty);
}
}  //  @jve:decl-index=0:visual-constraint="10,10"
