package UI;

import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JLabel;
import java.awt.Rectangle;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.math.BigInteger;
import java.net.ServerSocket;
import java.net.Socket;
import java.rmi.UnknownHostException;
import java.util.LinkedList;

import javax.swing.JTextArea;

import SecureProtocol.BinHDOTServer;

import DB.serverImages;

public class ServerUI extends JFrame {

	private static final long serialVersionUID = 1L;
	private JPanel jContentPane = null;
	private JLabel jLabel = null;
	private JTextArea jTextArea = null;
	String[] imageNameList=null;  
	LinkedList<boolean[]> imageVecList=null;
	int[] imageThresholdList=null;
	ServerSocket serverSock=null;
	Socket connection=null;
	BufferedReader br=null;
	BufferedWriter bw=null;
	int port=6086;
	String text;  


	/**
	 * This is the default constructor
	 */
	public ServerUI() {
		super();
		initialize();
	}
	
	public void start() {
		this.setVisible(true);
		readDB();
		text="Reading database";
		jTextArea.setText(text);
		startConnection();
		text=text+"\nStart connection and waiting for requests\n";
		jTextArea.setText(text);
		BinHDOTServer server=new BinHDOTServer(this.getJTextArea(),br,bw,imageVecList,imageThresholdList);
		server.preprocessing();
		LinkedList<BigInteger> l=server.executeAfterPreprocessing();
		text=text+"Results:\n";
		boolean noImageFound=true;
		for(int i=0; i<l.size(); i++) {
			if(l.get(i).compareTo(BigInteger.ONE)==0) {
				noImageFound=false;
				String img=imageNameList[i];
				text=text+"Image: "+img+"\n";
				jTextArea.setText(text);
				ImageDialog id=new ImageDialog(this);
		        id.setImage("./DB/"+img+".jpeg", "img");
			}
		}
		if(noImageFound) {
			text=text+"No match\n";
			jTextArea.setText(text);
		}
		text=text+"Finish\n";
		jTextArea.setText(text);
	}
	
	private void startConnection() {
		try {
			serverSock=new ServerSocket(port);
			connection=serverSock.accept();
			connection.setTcpNoDelay(true);
			br=new BufferedReader(new InputStreamReader(connection.getInputStream()));
			bw=new BufferedWriter(new OutputStreamWriter(connection.getOutputStream()));
		}
		catch(UnknownHostException uhe) {
			uhe.printStackTrace();
			return;
		}
		catch(IOException ioe) {
			ioe.printStackTrace();
			return;
		}
	}
	
	private void readDB() {
		LinkedList<String> tempList=null;
		tempList=serverImages.readFileDB("./DB/imageList.txt");
		
		for(int i=tempList.size()-1; i>=0; i--) {
			String s=tempList.get(i);
			if(s.length()<900) tempList.remove(i);
		}
		
		if(!tempList.isEmpty()) {
			imageNameList=new String[tempList.size()];
			imageVecList=new LinkedList<boolean[]>();
			imageThresholdList=new int[tempList.size()];
		} else {
			return;
		}
		
		for(int i=0; i<tempList.size(); i++) {
			boolean image[]=new boolean[900];
			String s=tempList.get(i);
			int space1=s.indexOf(' ');
			
			if(s.length()<900) continue;
			imageNameList[i]=new String(s.substring(0, space1));
			s=s.substring(space1+1);
			int space2=s.indexOf(' ');
			for(int j=0;j<space2;j++) {
				if(s.charAt(j)=='0') image[j]=false;
				if(s.charAt(j)=='1') image[j]=true;
			}
			
			imageVecList.add(image);
			
			s=s.substring(space2+1);
			imageThresholdList[i]=Integer.valueOf(s);
		}
		
	}

	/**
	 * This method initializes this
	 * 
	 */
	private void initialize() {
		this.setSize(297, 413);
		this.setContentPane(getJContentPane());
		this.setTitle("Server");
		this.addWindowListener(new java.awt.event.WindowAdapter() {
			public void windowClosing(java.awt.event.WindowEvent e) {
            System.exit(-1);
        	}
        });
		
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jLabel = new JLabel();
			jLabel.setBounds(new Rectangle(12, 31, 70, 19));
			jLabel.setText("Status:");
			jContentPane = new JPanel();
			jContentPane.setLayout(null);
			jContentPane.add(jLabel, null);
			jContentPane.add(getJTextArea(), null);
		}
		return jContentPane;
	}

	/**
	 * This method initializes jTextArea	
	 * 	
	 * @return javax.swing.JTextArea	
	 */
	private JTextArea getJTextArea() {
		if (jTextArea == null) {
			jTextArea = new JTextArea();
			jTextArea.setBounds(new Rectangle(12, 62, 254, 271));
		}
		return jTextArea;
	}
	
	public static void main(String[] args) {

        ServerUI sui=new ServerUI();
        sui.start();
	}


}  //  @jve:decl-index=0:visual-constraint="10,10"
